@@ -59,7 +59,7 @@ namespace opensaml {
         class SAML_DLLLOCAL ChainingMetadataProvider
             : public DiscoverableMetadataProvider, public ObservableMetadataProvider, public ObservableMetadataProvider::Observer {
         public:
-            ChainingMetadataProvider(const xercesc::DOMElement* e=nullptr);
+            ChainingMetadataProvider(const xercesc::DOMElement* e=nullptr, bool deprecationSupport=true);
             virtual ~ChainingMetadataProvider();
     
             using MetadataProvider::getEntityDescriptor;
@@ -98,13 +98,25 @@ namespace opensaml {
             }
 
             void onEvent(const ObservableMetadataProvider& provider) const {
-                // Reset the cache tag for the feed.
                 Lock lock(m_trackerLock);
-                SAMLConfig::getConfig().generateRandomBytes(m_feedTag, 4);
-                m_feedTag = SAMLArtifact::toHex(m_feedTag);
+                if (dynamic_cast<const DiscoverableMetadataProvider*>(&provider)) {
+                    // Reset the cache tag for the feed.
+                    SAMLConfig::getConfig().generateRandomBytes(m_feedTag, 4);
+                    m_feedTag = SAMLArtifact::toHex(m_feedTag);
+                }
                 emitChangeEvent();
             }
 
+            void onEvent(const ObservableMetadataProvider& provider, const EntityDescriptor& entity) const {
+                Lock lock(m_trackerLock);
+                if (dynamic_cast<const DiscoverableMetadataProvider*>(&provider)) {
+                    // Reset the cache tag for the feed.
+                    SAMLConfig::getConfig().generateRandomBytes(m_feedTag, 4);
+                    m_feedTag = SAMLArtifact::toHex(m_feedTag);
+                }
+                emitChangeEvent(entity);
+            }
+
         protected:
             void generateFeed() {
                 // No-op.
@@ -112,8 +124,8 @@ namespace opensaml {
 
         private:
             bool m_firstMatch;
-            mutable auto_ptr<Mutex> m_trackerLock;
-            auto_ptr<ThreadKey> m_tlsKey;
+            scoped_ptr<Mutex> m_trackerLock;
+            scoped_ptr<ThreadKey> m_tlsKey;
             mutable ptr_vector<MetadataProvider> m_providers;
             mutable set<tracker_t*> m_trackers;
             static void tracker_cleanup(void*);
@@ -153,9 +165,9 @@ namespace opensaml {
             map<const XMLObject*,const MetadataProvider*> m_objectMap;
         };
 
-        MetadataProvider* SAML_DLLLOCAL ChainingMetadataProviderFactory(const DOMElement* const & e)
+        MetadataProvider* SAML_DLLLOCAL ChainingMetadataProviderFactory(const DOMElement* const & e, bool deprecationSupport)
         {
-            return new ChainingMetadataProvider(e);
+            return new ChainingMetadataProvider(e, deprecationSupport);
         }
 
         static const XMLCh _MetadataProvider[] =    UNICODE_LITERAL_16(M,e,t,a,d,a,t,a,P,r,o,v,i,d,e,r);
@@ -176,34 +188,40 @@ void ChainingMetadataProvider::tracker_cleanup(void* ptr)
     }
 }
 
-ChainingMetadataProvider::ChainingMetadataProvider(const DOMElement* e)
-    : ObservableMetadataProvider(e), m_firstMatch(true), m_trackerLock(Mutex::create()), m_tlsKey(ThreadKey::create(tracker_cleanup)),
-        m_log(Category::getInstance(SAML_LOGCAT ".Metadata.Chaining"))
+ChainingMetadataProvider::ChainingMetadataProvider(const DOMElement* e, bool deprecationSupport)
+    : MetadataProvider(nullptr), ObservableMetadataProvider(e),
+        m_firstMatch(true), m_trackerLock(Mutex::create()), m_tlsKey(ThreadKey::create(tracker_cleanup)),
+        m_log(Category::getInstance(SAML_LOGCAT ".MetadataProvider.Chaining"))
 {
     if (XMLString::equals(e ? e->getAttributeNS(nullptr, precedence) : nullptr, last))
         m_firstMatch = false;
 
-    e = XMLHelper::getFirstChildElement(e, _MetadataProvider);
+    e = XMLHelper::getFirstChildElement(e);
     while (e) {
-        string t = XMLHelper::getAttrString(e, nullptr, _type);
-        if (!t.empty()) {
-            try {
-                m_log.info("building MetadataProvider of type %s", t.c_str());
-                auto_ptr<MetadataProvider> provider(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), e));
-                ObservableMetadataProvider* obs = dynamic_cast<ObservableMetadataProvider*>(provider.get());
-                if (obs)
-                    obs->addObserver(this);
-                m_providers.push_back(provider.get());
-                provider.release();
-            }
-            catch (std::exception& ex) {
-                m_log.error("error building MetadataProvider: %s", ex.what());
-            }
+        if (!XMLString::equals(_MetadataProvider, e->getLocalName())) {
+            auto_ptr_char name(e->getLocalName());
+            m_log.error("MetadataProvider child element of type %s ignored", name.get());
         }
         else {
-            m_log.error("MetadataProvider element missing type attribute");
+            string t = XMLHelper::getAttrString(e, nullptr, _type);
+            if (!t.empty()) {
+                try {
+                    m_log.info("building MetadataProvider of type %s", t.c_str());
+                    auto_ptr<MetadataProvider> provider(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), e, deprecationSupport));
+                    ObservableMetadataProvider* obs = dynamic_cast<ObservableMetadataProvider*>(provider.get());
+                    if (obs)
+                        obs->addObserver(this);
+                    m_providers.push_back(provider.get());
+                    provider.release();
+                } catch (std::exception& ex) {
+                    m_log.error("error building MetadataProvider: %s", ex.what());
+                }
+            }
+            else {
+                m_log.error("MetadataProvider element missing type attribute");
+            }
         }
-        e = XMLHelper::getNextSiblingElement(e, _MetadataProvider);
+        e = XMLHelper::getNextSiblingElement(e);
     }
 }
 
